/*************************************************************************
 *
 *  An implementation of Fortune's algorithm
 *  Copyright (C) 2012-2014 Jussi M. Hukkanen
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************
 *
 *  dotvga.h: header file for graphics output using svgalib
 *
 ************************************************************************/

#ifndef _DOTVGA_H_INCLUDED
#define _DOTVGA_H_INCLUDED

#ifndef VALGRIND
#include <vga.h>
#include <vgagl.h>
#endif

#include <stdint.h>
#include <math.h>	/* declares lrintf() */


#define WINDOW_WIDTH  (WINDOW_RIGHT - WINDOW_LEFT)
#define WINDOW_HEIGHT (WINDOW_TOP   - WINDOW_BOTTOM)

#define PIXEL_X_(x) (((x) - WINDOW_LEFT) * SCREEN_WIDTH ) / WINDOW_WIDTH
#define PIXEL_Y_(y) ((WINDOW_TOP - (y))  * SCREEN_HEIGHT) / WINDOW_HEIGHT
#define WORLD_X_(x) WINDOW_LEFT  + (((x) * WINDOW_WIDTH ) / SCREEN_WIDTH )
#define WORLD_Y_(y) WINDOW_TOP   - (((y) * WINDOW_HEIGHT) / SCREEN_HEIGHT)

#if 1 /* floating-point coordinates */

#define WINDOW_LEFT   (-4.0 / 3.0)
#define WINDOW_RIGHT  ( 4.0 / 3.0)
#define WINDOW_TOP    ( 1.0)
#define WINDOW_BOTTOM (-1.0)

#define PIXEL_X(x) lrintf(PIXEL_X_(x))
#define PIXEL_Y(y) lrintf(PIXEL_Y_(y))
#define WORLD_X(x) ((x < 0) ? WORLD_X_(x - 0.5) : WORLD_X_(x + 0.5))
#define WORLD_Y(y) ((y < 0) ? WORLD_Y_(y - 0.5) : WORLD_Y_(y + 0.5))

#else /* integer coordinates */

#define WINDOW_LEFT   (  0)
#define WINDOW_RIGHT  (320)
#define WINDOW_TOP    (240)
#define WINDOW_BOTTOM (  0)

#define PIXEL_X(x) PIXEL_X_(x)
#define PIXEL_Y(y) PIXEL_Y_(y)
#define WORLD_X(x) WORLD_X_(x)
#define WORLD_Y(y) WORLD_Y_(y)

#endif


int vga_accel_caps;
char get_vga_accel_caps_called;


#ifdef VALGRIND

typedef char GraphicsContext;

#define SCREEN_MODE_DEFAULT 1
#define VALGRIND_WIDTH_DEFAULT  320
#define VALGRIND_HEIGHT_DEFAULT 200

#define ACCELFLAG_DRAWHLINELIST 0
#define ACCELFLAG_SETFGCOLOR    0

char vga_getch(void);
char vga_getkey(void);
void gl_setcontext(GraphicsContext *);
int vga_accel(unsigned, ...);

#define vga_disabledriverreport() { }
#define gl_setpixel(x, y, c) { }
#define gl_hline(x1, y, x2, c) { }
#define gl_line(x1, y1, x2, y2, c) { }

#endif

#ifndef SCREEN_MODE_DEFAULT
#define SCREEN_MODE_DEFAULT G320x200x256
#endif

int SCREEN_MODE;
int SCREEN_WIDTH, SCREEN_HEIGHT;


GraphicsContext *scr_physical;
GraphicsContext *scr_virtual;
GraphicsContext *scr_frame;


#define BRIGHT_COLOUR(X) ((X) | 8)


int init_vga(uint8_t);
#define INIT_VGA_DOUBLE_BUFFER 0x01	 /* create scr_frame */
#define INIT_VGA_DRAW_FRAME	   0x02  /* draw blue frame around screen */

char exit_vga(void);
char get_vga_accel_caps(void);
void report_vga_accel_caps(void);
char *list_vga_modes(void);
char find_vga_mode(char *);

#define dot_getch	vga_getch
#define dot_getkey	vga_getkey

char copy_screen(GraphicsContext *, GraphicsContext *);
#define blit_screen(S) copy_screen(S, scr_physical)

char fill_screen(GraphicsContext *, uint8_t);

char plot_dot_ex(int, int, uint8_t, uint8_t, uint8_t);
#define PLOT_DOT_HOLLOW        0x01  /* circle instead of dot */
#define PLOT_DOT_PHYSICAL      0x02  /* use scr_physical (or scr_frame)
                                          instead of scr_virtual */

char draw_boundary_ex(float, float, float, float, uint8_t, uint8_t, float);
#define DRAW_BOUNDARY_RELATIVE 0x01  /* x2,y2 is relative to x1,y1 */
#define DRAW_BOUNDARY_PHYSICAL 0x02  /* use scr_physical (or scr_frame)
                                          instead of scr_virtual */
#define draw_boundary(X1, Y1, X2, Y2, FLAGS, COLOUR) \
     draw_boundary_ex(X1, Y1, X2, Y2, FLAGS, COLOUR, 1.0)

char save_pbm_screenshot(char *, GraphicsContext *, uint8_t);

#endif /* _DOTVGA_H_INCLUDED */
