/*************************************************************************
 *
 *  Routine for converting Delaunay triangulations into Gabriel graphs
 *  Copyright (C) 2013-2014 Jussi M. Hukkanen
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************
 *
 *  gabriel.c: main module
 *
 ************************************************************************/

#include "gabriel.h"

#ifndef FORTUNE_NO_DELAUNAY


/* Delaunay_to_Gabriel()

	finds out which sites in the given
	  set of triangles are not Gabriel
	  neighbours;
	the extra field in each connection is
	  pointed to a char that is nonzero
	  if the connection connects two
	  Gabriel neighbours */

char Delaunay_to_Gabriel(triangle_t *t_in, int t_size, connect_t *c_in, int c_size) {

	char *gab;
	int i;
	uint8_t c, mask;

	if (t_in == NULL)
		return RET_FORTUNE_PARAM_ERROR;
	if (c_in == NULL)
		return RET_FORTUNE_PARAM_ERROR;

	for (i = 0; i < c_size; i++)
		if (c_in[i].extra != NULL)
			return RET_FORTUNE_PARAM_ERROR;

	gab = (char *)malloc(c_size);

	if (gab == NULL)
		return RET_FORTUNE_NO_MEM;

	for (i = 0; i < c_size; i++) {
		gab[i] = 1;
		c_in[i].extra = &gab[i];
	}

	for (i = 0; i < t_size; i++) {
		for (c = 0, mask = 0x01; c < 3; c++, mask <<= 1)
			if (t_in[i].shape & mask) {
				gab = (char *)t_in[i].conn[c]->extra;
				*gab = 0;
			}
	}

	return RET_FORTUNE_OK;
}


/* Gabriel_cleanup()

	destroys the data structures
	  attached to the connections
	  in input */

char Gabriel_cleanup(connect_t *input) {

	if (input == NULL)
		return RET_FORTUNE_OK;

	if (input[0].extra != NULL)
		free(input[0].extra);

	return RET_FORTUNE_OK;
}

#endif
