/*************************************************************************
 *
 *  An implementation of Fortune's algorithm
 *  Copyright (C) 2012-2014 Jussi M. Hukkanen
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************
 *
 *  opts.c: command line option parser shared by several programs
 *
 ************************************************************************/

#include "opts.h"


/* make_opt_string()

	translates a table of long options into a
	  string of short option characters to be
	  used with getopt() or getopt_long();
	table is the array that contains the
	  available options; the val field in each
	  item should be the short option character
	  corresponding to that option;
	the last item in table must have NULL as
	  the name field */

char *make_opt_string(struct option *table) {

	int i, c;
	int len;
	char *target;

	opterr = 0;  /* suppress error messages from getopt_long() */

	for (i = len = 0; ; i++) {

		if (table[i].name == NULL)
			break;

		switch (table[i].has_arg) {
			case optional_argument:	len++;
			case required_argument:	len++;
			case no_argument:		len++;
		};
	}

	target = malloc(len + 1);
	if (target == NULL)
		return NULL;

	for (i = c = 0; c < len; i++) {

		if (table[i].name == NULL)
			break;

		target[c++] = table[i].val;

		if (c < len)
			if (table[i].has_arg != no_argument)
				target[c++] = ':';
			if (table[i].has_arg == optional_argument)
				target[c++] = ':';
	}

	target[c] = '\0';

	return target;
}


/* get_input_file_name()

	returns the first argument in the argv
	  array that is recognised as the name
	  of a file that can be opened;
	returns NULL if no such argument
	  is found */

char *get_input_file_name(int argc, char *argv[]) {

	struct stat statb;
	int a;
	char *n;

	for (a = 1; a < argc; a++) {

		n = argv[a];

		if (*n == '-')
			continue;

		if (access(n, F_OK | R_OK) == 0) {	/* exists and can be read */
			stat(n, &statb);
			if (!S_ISDIR(statb.st_mode))	/* is not a directory */
				return n;
		}
	}

	return NULL;
}

