/*************************************************************************
 *
 *  An implementation of Fortune's algorithm
 *  Copyright (C) 2012-2014 Jussi M. Hukkanen
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************
 *
 *  rnd.h: pseudo-random number generator
 *
 ************************************************************************/


#ifndef _RND_H_INCLUDED
#define _RND_H_INCLUDED

#include <stdlib.h>


/* an application could #define USE_RAND48
     and the ISO C standard rand() function
     would be replaced by the SVID PRNGs
     drand48() and lrand48();
   unfortunately, the SVID functions are not
     available in Windows */

#ifdef USE_RAND48
#ifdef TARGET_WINDOWS
#undef USE_RAND48
#endif
#endif

#ifdef USE_RAND48

#define RND_INT(MAX)	(lrand48() % (MAX))
#define RND_BITS(MASK)	(lrand48() & (MASK))
#define RND_FLOAT(MAX)	(drand48() * (MAX))
#define RND_FLOAT_1		drand48()

#define RND_SEED(VAL)	{ srand48(VAL); }

#else

#define RND_INT(MAX)	(rand() % (MAX))
#define RND_BITS(MASK)	(rand() & (MASK))
#define RND_FLOAT(MAX)	(rand() * ((MAX) / RAND_MAX))
#define RND_FLOAT_1		RND_FLOAT(1.0)

#define RND_SEED(VAL)	{ srand(VAL); }

#endif


#endif /* _RND_H_INCLUDED */
